%language "c"
%param {yyscan_t scanner}
%define api.prefix {conf}
%define api.pure full
%define lr.type ielr
%define parse.lac full
%define parse.error verbose
//%define parse.trace
%locations
//%no-lines

// Get name of initial parsed file
%initial-action {@$.filename=confget_extra(scanner)->state.curdir+confget_extra(scanner)->state.filename;}

%{
#include <inttypes.h>
#include "parser.h"
#include "object.h"
#include "globals.h"
// We can't include lexical.h before grammatical.h, because of strange errors, but grammatical.h only needs definition of yyscan_t
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

#include "grammatical.h"
#include "lexical.h"

inline void conferror(const YYLTYPE* locp, yyscan_t sc, const char* str)
{
 COUT(ERROR)<<std::endl<<" Grammatical parser: "<<str<<std::endl<<"  in file "<<locp->filename;
 if(locp->first_line==locp->last_line) COUT(ERROR)<<" at line "<<locp->last_line<<", from position "<<locp->first_column<<" to "<<locp->last_column<<std::endl;
 else COUT(ERROR)<<" from line "<<locp->first_line<<", position "<<locp->first_column<<" to line "<<locp->last_line<<", position "<<locp->last_column<<std::endl;
 for(const auto &i:locp->incstack) COUT(ERROR)<<"   included from "<<i.filename<<" at line "<<i.line<<", position "<<i.column<<std::endl;
}
inline void conferror(const YYLTYPE& locp, yyscan_t sc, const std::string& str)
{
 conferror(&locp,sc,str.c_str());
}

#define EADD(to,from) {(to)->splice((to)->end(),*(from)); delete (from);}
#define EADDV(to,from) {(to).splice((to).end(),*(from)); delete (from);}
#define I(i) {(i)=new ExecExpr;}
#define G_Vars (*(confget_extra(scanner)->vars))
#define G_ToPrint (confget_extra(scanner)->toprint)
#define G_ToSave (confget_extra(scanner)->tosave)
%}

%union
{
 bool b;
 int64_t i;
 double r;
 std::string* str;
 ExecExpr* expr;
}

%token ASSIGN OBRACE CBRACE ENDL DELIM
%token DIR_USE DIR_INCLUDEPATH DIR_MODULEPATH DIR_AS

%token <r> REAL
%token <b> BOOL
%token <i> INTEGER
%token <str> NAME
%token <str> IDENTIFIER
%token <str> STRING

%left DELIM
%precedence ASSIGN
%precedence IDENTIFIER /* low-priority for just identifier */
%left '-' '+'
%left '*' '/'
%precedence UNARY /* negation--unary minus */
%right '^'        /* exponentiation */
%left '.'
%precedence OBRACE CBRACE

%type <expr> longidentifier
%type <expr> expression
%type <expr> object
%type <expr> pair
%type <expr> list

%destructor {} REAL INTEGER BOOL
%destructor {delete $$;} <*>

%%
input:
        %empty          {COUT(DEBUG)<<"Empty input\n";}
        | input line    {COUT(DEBUG)<<" input line\n";}
        | input dirline {COUT(DEBUG)<<" input dirline\n";}
        ;

dirline:
        DIR_INCLUDEPATH STRING          {COUT(DEBUG)<<" DIR_INCLUDEPATH STRING\n"; confget_extra(scanner)->includedirs+=":"+*$2+(('/'==$2->back())?"":"/"); delete $2;}
        | DIR_MODULEPATH STRING         {COUT(DEBUG)<<" DIR_MODULEPATH STRING\n"; confget_extra(scanner)->moduledirs+=":"+*$2+(('/'==$2->back())?"":"/"); delete $2;}
        | DIR_USE STRING                {COUT(DEBUG)<<" DIR_USE STRING\n";
                                         int ret=LoadModule(*$2,&confget_extra(scanner)->moduledirs);
                                         if(0!=ret) conferror(@1,scanner," fail to load module "+(*$2));
                                         delete $2;
                                         if(0!=ret) YYABORT;
                                        }
        | DIR_USE STRING DIR_AS STRING  {COUT(DEBUG)<<" DIR_USE STRING DIR_AS STRING\n";
                                         int ret=LoadModule(*$2,&confget_extra(scanner)->moduledirs,*$4);
                                         if(0!=ret) conferror(@1,scanner," fail to load module "+(*$2));
                                         delete $2; delete $4;
                                         if(0!=ret) YYABORT;
                                        }

line:
        NAME ASSIGN object ENDL                 {COUT(DEBUG)<<" NAME ASSIGN object ENDL\n"; EADDV(G_Vars[*$1],$3); delete $1;}
        | NAME ASSIGN list ENDL                 {COUT(DEBUG)<<" NAME ASSIGN list ENDL\n"; $3->push_front(SEBList(@3)); $3->push_back(SEEList(@4)); EADDV(G_Vars[*$1],$3); delete $1;}
        | NAME OBRACE list CBRACE ENDL          {COUT(DEBUG)<<" NAME OBRACE list CBRACE ENDL\n";
                                                 tolower($1);
                                                 $3->push_front(SEBList(@2));
                                                 $3->push_back(SEEList(@4));
                                                 if(*$1=="save") {G_ToSave->emplace_back(); EADDV(G_ToSave->back(),$3);}
                                                 else if(*$1=="print") {G_ToPrint->emplace_back(); EADDV(G_ToPrint->back(),$3);}
                                                 else
                                                 {
                                                  conferror(@1,scanner,"unknown statement "+(*$1));
                                                  delete $1; delete $3;
                                                  YYABORT;
                                                 }
                                                 delete $1;}
        | NAME OBRACE object CBRACE ENDL        {COUT(DEBUG)<<" NAME OBRACE object CBRACE ENDL\n";
                                                 tolower($1);
                                                 $3->push_front(SEBList(@2));
                                                 $3->push_back(SEEList(@4));
                                                 if(*$1=="save") {G_ToSave->emplace_back(); EADDV(G_ToSave->back(),$3);}
                                                 else if(*$1=="print") {G_ToPrint->emplace_back(); EADDV(G_ToPrint->back(),$3);}
                                                 else
                                                 {
                                                  conferror(@1,scanner,"unknown statement "+(*$1));
                                                  delete $1; delete $3;
                                                  YYABORT;
                                                 }
                                                 delete $1;}
        ;

list:
        object object           {COUT(DEBUG)<<" object object\n"; I($$); EADD($$,$1); EADD($$,$2);}
        | list object           {COUT(DEBUG)<<" list object\n"; I($$); EADD($$,$1); EADD($$,$2);}
        ;

pair:
        IDENTIFIER ASSIGN object        {COUT(DEBUG)<<" IDENTIFIER ASSIGN object "<<*$1<<"\n"; I($$); EADD($$,$3); $$->push_back(SEMKPair($1,@1)); delete $1;}
        ;

object:
        STRING                  {COUT(DEBUG)<<" STRING\n"; I($$); $$->push_back(StackElem(new ObjectString($1),@1)); delete $1;}
        | BOOL                  {COUT(DEBUG)<<" BOOL\n"; I($$); $$->push_back(StackElem(new ObjectBool($1),@1));}
        | OBRACE list CBRACE    {COUT(DEBUG)<<" OBRACE list CBRACE\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$2); $$->push_back(SEEList(@3));}
        | expression %prec ASSIGN           {COUT(DEBUG)<<" expression\n"; I($$); EADD($$,$1);}
        | pair                  {COUT(DEBUG)<<" pair\n"; I($$); EADD($$,$1);}
        | OBRACE object CBRACE  {COUT(DEBUG)<<" OBRACE object CBRACE\n"; I($$); EADD($$,$2);}
        | object DELIM          {COUT(DEBUG)<<" OBJECT DELIM\n"; I($$); EADD($$,$1);}
        ;

longidentifier:
        IDENTIFIER '.' IDENTIFIER               {COUT(DEBUG)<<" IDENTIFIER DOT IDENTIFIER\n"; I($$); $$->push_front(SEBList(@1)); $$->push_back(SEVar($1,@1)); $$->push_back(SEObj(new ObjectString($3),@3)); $$->push_back(SEEList(@1)); $$->push_back(SEFunc("GET",@2)); delete $1; delete $3;}
        | longidentifier '.' IDENTIFIER         {COUT(DEBUG)<<" longidentifier '.' IDENTIFIER\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$1); $$->push_back(SEObj(new ObjectString($3),@3)); $$->push_back(SEEList(@1)); $$->push_back(SEFunc("GET",@2)); delete $3;}

expression:
        IDENTIFIER                              {COUT(DEBUG)<<" IDENTIFIER\n"; I($$); $$->push_back(SEVar($1,@1)); delete $1;}
        | IDENTIFIER OBRACE object CBRACE       {COUT(DEBUG)<<" IDENTIFIER OBRACE object CBRACE\n"; I($$); $$->push_front(SEBList(@2)); EADD($$,$3); $$->push_back(SEEList(@4)); $$->push_back(SEFunc($1,@1)); delete $1;}
        | IDENTIFIER OBRACE list CBRACE         {COUT(DEBUG)<<" IDENTIFIER OBRACE list CBRACE\n"; I($$); $$->push_front(SEBList(@2)); EADD($$,$3); $$->push_back(SEEList(@4)); $$->push_back(SEFunc($1,@1)); delete $1;}
        | longidentifier                        {COUT(DEBUG)<<" longidentifier\n"; I($$); EADD($$,$1);}
        | REAL                                  {COUT(DEBUG)<<" REAL\n"; I($$); $$->push_front(StackElem(new ObjectReal($1),@1));}
        | INTEGER                               {COUT(DEBUG)<<" INTEGER\n"; I($$); $$->push_front(StackElem(new ObjectInt($1),@1));}
        | expression '-' expression             {COUT(DEBUG)<<" -\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$1); EADD($$,$3); $$->push_back(SEEList(@3)); $$->push_back(SEFunc("SUB",@2));}
        | expression '+' expression             {COUT(DEBUG)<<" +\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$1); EADD($$,$3); $$->push_back(SEEList(@3)); $$->push_back(SEFunc("ADD",@2));}
        | expression '/' expression             {COUT(DEBUG)<<" /\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$1); EADD($$,$3); $$->push_back(SEEList(@3)); $$->push_back(SEFunc("DIV",@2));}
        | expression '*' expression             {COUT(DEBUG)<<" *\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$1); EADD($$,$3); $$->push_back(SEEList(@3)); $$->push_back(SEFunc("MUL",@2));}
        | expression '^' expression             {COUT(DEBUG)<<" ^\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$1); EADD($$,$3); $$->push_back(SEEList(@3)); $$->push_back(SEFunc("POW",@2));}
        | '-' expression %prec UNARY            {COUT(DEBUG)<<" unary -\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$2); $$->push_back(SEEList(@2)); $$->push_back(SEFunc("NEG",@1));}
        | '+' expression %prec UNARY            {COUT(DEBUG)<<" unary +\n"; I($$); $$->push_front(SEBList(@1)); EADD($$,$2); $$->push_back(SEEList(@2)); $$->push_back(SEFunc("POS",@1));}
        | OBRACE expression CBRACE              {COUT(DEBUG)<<" OBRACE expression CBRACE\n"; I($$); EADD($$,$2);}
        ;
